% ScalarProblem_simulationResults.m
%
% Generates simulation results for scalar problem (Sec. III-A) presented 
% in Fig. 2.
%
% Simulation of the variance iteration (21) for Example 1 and different
% values of the sending threshold \delta.
%
% --
% Supplementary material to
%
% S. Trimpe, R. D'Andrea, "Event-Based State Estimation with Variance-Based 
% Triggering", IEEE Transactions on Automatic Control, Special Issue on 
% Control of Cyber-Physical Systems.
%
% Copyright:
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% strimpe(at)tuebingen.mpg.de
% March 2014
%

% Revision history
% [Dec 12, ST]   first version
% [Dec 13, ST]   changed scaling of figures
% [Dec 13, ST]   adapt equation numbers etc. to revised version of the 
%                paper
% [10.03.14, ST] adapt to revised version of the paper

clear;

disp(' ');
disp('## Section III-A, scalar examples.');
disp('## Results from simulating (21) for scalar examples (Fig. 2).');
disp(' ');


%% Configuration
% Uncomment to use latex interpreter.
%set(0,'defaulttextinterpreter','latex');    % set text interpreter

% Line color in plots.
lineCol = 'k';      % default
deltaCol = 'k';     % threshold delta

% Line width
deltaWidth = 0.8;     % threshold delta

% Dot size in plots
markersize = 8;

% subplot settings
% (Leave 10th subplot row empty to make space for xlabel.)
subplotSetttingT = {10,1,1:6};      % top figure
subplotSetttingB = {10,1,7:9};     % bottom figure


%% parameters
% Problem parameters: [a c delta]. Q and R are assumed 1 w.l.o.g.
param(1,:) = [1.2 1 0.2];       % 5-periodic solution 
param(2,:) = [1.2 1 3];         % 3-periodic solution 
param(3,:) = [1.2 1 9.6167];    % 19-periodic

% periods
N = [5 3 19];

% time horizon for simulation
K = 5*N;            % 5 times period length


%% Perform simulation

% Variable assignments 
nEx = size(param,1);    % number of examples

for i=1:nEx
    pp{i} = zeros(K(i)+1,1);        % vector to hold result for p(k)
    gamma{i} = zeros(K(i),1);       % transmit decision: gamma(k)
    kk{i} = 0:1:K(i);               % iteration index k
end;

% Assign initial conditions: pbar; compute sending threshold
for i=1:nEx
    a = param(i,1);
    c = param(i,2);
    d = param(i,3);
    
    % Start iteration at the solution of the DARE.
    p_bar{i} = dare(a,c,1,1);
    pp{i}(1) = p_bar{i};
    
    % Compute sending threshold (used only for illustration in figures).
    p2(i) = pp{i}(1) + d;
end;

% Simulate each example
for i=1:nEx
    a = param(i,1);
    c = param(i,2);
    d = param(i,3);
    
    for k=1:K(i)
        % Iterate: p(k+1) = H(p(k)), equation (21) (scalar).
        % Also returns \gamma(k).
        [pp{i}(k+1),gamma{i}(k)] = map_H(pp{i}(k),a,c,1,1,p_bar{i},d);
    end;
end;


%% Plot1: 5-periodic solution
fig1 = figure;
iEx = 1;        % example number
set(gcf,'Name',['(b) delta = ',num2str(param(iEx,3))]);

% pk iterates
subplot(subplotSetttingT{1},subplotSetttingT{2},subplotSetttingT{3});
plot(kk{iEx},pp{iEx}(:),[lineCol,'.-'],'Markersize',markersize);
hold on;
plot([kk{iEx}(1),kk{iEx}(end)],[p2(iEx) p2(iEx)],[deltaCol,'--'],'Linewidth',deltaWidth);

ylabel('$p(k)$');
grid;
set(gca,'xlim',[kk{i}(1),kk{iEx}(end)]);
set(gca,'ylim',[min(pp{iEx}(:))-0.5,max(pp{iEx}(:))+0.5]);
set(gca,'XTick',[0:N(1):kk{iEx}(end)]);
set(gca,'XTickLabel','');
set(gca,'YTick',round(100*[pp{iEx}(26) pp{iEx}(24)])/100);
set(gca,'YTickLabel',{num2str(round(100*pp{iEx}(26))/100),num2str(round(100*pp{iEx}(24))/100)});

% transmit decisions gamma
subplot(subplotSetttingB{1},subplotSetttingB{2},subplotSetttingB{3});
stem(kk{iEx}(1:end-1),gamma{iEx}(:),[lineCol,'.'],'Markersize',markersize);

xlabel('Iteration $k$');
ylabel('$\gamma(k)$');
grid;
set(gca,'xlim',[kk{iEx}(1),kk{iEx}(end)]);
set(gca,'ylim',[-0.2 1.2]);
set(gca,'XTick',[0:N(1):kk{iEx}(end)]);
set(gca,'YTick',[0 1]);


%% Plot2: 3-periodic solution
fig2 = figure;
iEx = 2;        % example number
set(gcf,'Name',['(a) delta = ',num2str(param(iEx,3))]);

% pk iterates
subplot(subplotSetttingT{1},subplotSetttingT{2},subplotSetttingT{3});
plot(kk{iEx},pp{iEx}(:),[lineCol,'.-'],'Markersize',markersize);
hold on;
plot([kk{iEx}(1),kk{iEx}(end)],[p2(iEx) p2(iEx)],[deltaCol,'--'],'Linewidth',deltaWidth);

ylabel('$p(k)$');
grid;
set(gca,'xlim',[kk{iEx}(1),kk{iEx}(end)]);
set(gca,'ylim',[min(pp{iEx}(:))-1,max(pp{iEx}(:))+1]);
set(gca,'XTick',[0:N(2):kk{iEx}(end)]);
set(gca,'XTickLabel','');
set(gca,'YTick',round( 100*[sort([pp{iEx}(end) pp{iEx}(end-1) pp{iEx}(end-2)]) ] )/100);

% transmit decisions gamma
subplot(subplotSetttingB{1},subplotSetttingB{2},subplotSetttingB{3});
stem(kk{iEx}(1:end-1),gamma{iEx}(:),[lineCol,'.'],'Markersize',markersize);

xlabel('Iteration $k$');
ylabel('$\gamma(k)$');
grid;
set(gca,'xlim',[kk{iEx}(1),kk{iEx}(end)]);
set(gca,'ylim',[-0.2 1.2]);
set(gca,'XTick',[0:N(2):kk{iEx}(end)]);
set(gca,'YTick',[0 1]);


%% Plot3: 19-periodic solution
fig3 = figure;
iEx = 3;        % example number
set(gcf,'Name',['(c) delta = ',num2str(param(iEx,3))]);

% pk iterates
subplot(subplotSetttingT{1},subplotSetttingT{2},subplotSetttingT{3});
plot(kk{iEx},pp{iEx}(:),[lineCol,'.-'],'Markersize',markersize);
hold on;
plot([kk{iEx}(1),kk{iEx}(end)],[p2(iEx) p2(iEx)],[deltaCol,'--'],'Linewidth',deltaWidth);

ylabel('$p(k)$');
grid;
set(gca,'xlim',[kk{iEx}(1),kk{iEx}(end)]);
set(gca,'ylim',[min(pp{iEx}(:))-1,max(pp{iEx}(:))+1]);
set(gca,'XTick',[0:N(3):kk{iEx}(end)]);
set(gca,'XTickLabel','');
set(gca,'YTick',round(100*[sort(pp{iEx}(49:53)')])/100);

% transmit decisions gamma
subplot(subplotSetttingB{1},subplotSetttingB{2},subplotSetttingB{3});
stem(kk{iEx}(1:end-1),gamma{iEx}(:),[lineCol,'.'],'Markersize',markersize);

xlabel('Iteration $k$');
ylabel('$\gamma(k)$');
grid;
set(gca,'xlim',[kk{iEx}(1),kk{iEx}(end)]);
set(gca,'ylim',[-0.2 1.2]);
set(gca,'XTick',[0:N(3):kk{iEx}(end)]);
set(gca,'YTick',[0 1]);



disp(' ');
disp('# Simulation results:');
disp(['The plots in Matlab Figures ',num2str(fig1),' to ',num2str(fig3),' are the results shown in Fig. 2 of the paper ']);
disp('"Event-Based State Estimation with Variance-Based Triggering".');
disp(' ');

